@@ -1,12 +1,12 @@
 
 /*
- *  $Id: expGuile.c,v 4.1 2005/01/01 00:12:33 bkorb Exp $
+ *  $Id: expGuile.c,v 4.4 2005/06/07 22:26:43 bkorb Exp $
  *  This module implements the expression functions that should
  *  be part of Guile.
  */
 
 /*
- *  AutoGen copyright 1992-2004 Bruce Korb
+ *  AutoGen copyright 1992-2005 Bruce Korb
  *
  *  AutoGen is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -21,11 +21,11 @@
  *  You should have received a copy of the GNU General Public License
  *  along with AutoGen.  See the file "COPYING".  If not,
  *  write to:  The Free Software Foundation, Inc.,
- *             59 Temple Place - Suite 330,
- *             Boston,  MA  02111-1307, USA.
+ *             51 Franklin Street, Fifth Floor,
+ *             Boston, MA  02110-1301, USA.
  */
 
-teGuileType
+LOCAL teGuileType
 gh_type_e( SCM typ )
 {
     if (gh_boolean_p( typ ))
@@ -64,7 +64,7 @@ gh_type_e( SCM typ )
     return GH_TYPE_UNDEFINED;
 }
 
-EXPORT SCM
+LOCAL SCM
 ag_scm_c_eval_string_from_file_line( tCC* pzExpr, tCC* pzFile, int line )
 {
     SCM port;
