@@ -1,12 +1,12 @@
 
 /*
- *  $Id: funcEval.c,v 1.49 2004/02/02 03:31:50 bkorb Exp $
+ *  $Id: funcEval.c,v 4.4 2005/06/07 22:26:43 bkorb Exp $
  *
  *  This module evaluates macro expressions.
  */
 
 /*
- *  AutoGen copyright 1992-2004 Bruce Korb
+ *  AutoGen copyright 1992-2005 Bruce Korb
  *
  *  AutoGen is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -21,16 +21,22 @@
  *  You should have received a copy of the GNU General Public License
  *  along with AutoGen.  See the file "COPYING".  If not,
  *  write to:  The Free Software Foundation, Inc.,
- *             59 Temple Place - Suite 330,
- *             Boston,  MA  02111-1307, USA.
+ *             51 Franklin Street, Fifth Floor,
+ *             Boston, MA  02110-1301, USA.
  */
 
-STATIC int exprType( char* pz );
+static int exprType( char* pz );
 
-EXPORT char*
+/* = = = START-STATIC-FORWARD = = = */
+/* static forward declarations maintained by :mkfwd */
+static int
+exprType( char* pz );
+/* = = = END-STATIC-FORWARD = = = */
+
+LOCAL char*
 resolveSCM( SCM s )
 {
-    static char z[32];
+    static char z[48];
     char*  pzRes = z;
 
     switch (gh_type_e( s )) {
@@ -99,7 +105,7 @@ resolveSCM( SCM s )
  *  It may need to be deallocated, so a boolean pointer is used
  *  to tell the caller.
  */
-EXPORT char*
+LOCAL char*
 evalExpression( ag_bool* pMustFree )
 {
     tTemplate*  pT      = pCurTemplate;
@@ -146,7 +152,7 @@ evalExpression( ag_bool* pMustFree )
                 /*
                  *  Emit only if found
                  */
-                return (char*)zNil;
+                return (char*)zDefaultNil;
 
             case (EMIT_IF_ABSENT | EMIT_ALWAYS):
                 /*
@@ -164,13 +170,13 @@ evalExpression( ag_bool* pMustFree )
             tSCC zBlock[] = "attempted to use block macro in eval expression";
 
             if ((code & EMIT_IF_ABSENT) != 0)
-                return (char*)zNil;
+                return (char*)zDefaultNil;
 
             if (  (pDef->valType != VALTYP_TEXT)
                && ((code & EMIT_PRIMARY_TYPE) == EMIT_VALUE)  ) {
                 fprintf( pfTrace, zTplWarn, pT->pzFileName, pMac->lineNo,
                          zBlock );
-                return (char*)zNil;
+                return (char*)zDefaultNil;
             }
 
             /*
@@ -191,7 +197,7 @@ evalExpression( ag_bool* pMustFree )
                 if (pDef->valType != VALTYP_TEXT) {
                     fprintf( pfTrace, zTplWarn, pT->pzFileName, pMac->lineNo,
                              zBlock );
-                    return (char*)zNil;
+                    return (char*)zDefaultNil;
                 }
 
                 *pMustFree = AG_TRUE;
@@ -208,7 +214,7 @@ evalExpression( ag_bool* pMustFree )
                 if (pDef->valType != VALTYP_TEXT) {
                     fprintf( pfTrace, zTplWarn, pT->pzFileName, pMac->lineNo,
                              zBlock );
-                    return (char*)zNil;
+                    return (char*)zDefaultNil;
                 }
 
                 pzText = pDef->val.pzText;
@@ -290,6 +296,52 @@ ag_scm_error_source_line( void )
     return SCM_UNDEFINED;
 }
 
+
+/*=gfunc emit
+ *
+ * what: emit the text for each argument
+ * general_use:
+ *
+ * exparg: alist, list of arguments to stringify and emit, , list
+ *
+ * doc:  Walk the tree of arguments, displaying the values of displayable
+ *       SCM types.
+=*/
+SCM 
+ag_scm_emit( SCM val )
+{
+    for (;;) {
+        if (val == SCM_UNDEFINED)
+            break;
+
+        if (SCM_NULLP( val ))
+            break;
+
+        if (gh_string_p( val )) {
+            fputs( (char*)ag_scm2zchars( val, "emit val" ), pCurFp->pFile );
+            fflush( pCurFp->pFile );
+            break;
+        }
+
+        switch (gh_type_e( val )) {
+        case GH_TYPE_LIST:
+        case GH_TYPE_PAIR:
+            ag_scm_emit( SCM_CAR( val ));
+            val = SCM_CDR( val );
+            continue;
+
+        default:
+            fputs( resolveSCM( val ), pCurFp->pFile );
+            fflush( pCurFp->pFile );
+            break;
+        }
+
+        break;
+    }
+
+    return SCM_UNDEFINED;
+}
+
 /*
  *  eval
  *
@@ -302,7 +354,7 @@ ag_scm_error_source_line( void )
  *  digest the string and return that.  Otherwise, just return
  *  the string.
  */
-EXPORT SCM
+LOCAL SCM
 eval( const char* pzExpr )
 {
     ag_bool allocated = AG_FALSE;
@@ -345,6 +397,7 @@ eval( const char* pzExpr )
 /*=macfunc EXPR
  *
  *  what:  Evaluate and emit an Expression
+ *  alias: + - + ? + % + ; + ( + . + '`' + '"' + "'" +
  *
  *  desc:
  *   This macro does not have a name to cause it to be invoked
@@ -368,7 +421,7 @@ mFunc_Expr( tTemplate* pT, tMacro* pMac )
 }
 
 
-STATIC int
+static int
 exprType( char* pz )
 {
     switch (*pz) {
