@@ -1,12 +1,12 @@
 
 /*
- *  $Id: funcFor.c,v 1.43 2004/10/11 23:30:00 bkorb Exp $
+ *  $Id: funcFor.c,v 4.5 2005/06/07 22:26:43 bkorb Exp $
  *
  *  This module implements the FOR text macro.
  */
 
 /*
- *  AutoGen copyright 1992-2004 Bruce Korb
+ *  AutoGen copyright 1992-2005 Bruce Korb
  *
  *  AutoGen is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -21,25 +21,35 @@
  *  You should have received a copy of the GNU General Public License
  *  along with AutoGen.  See the file "COPYING".  If not,
  *  write to:  The Free Software Foundation, Inc.,
- *             59 Temple Place - Suite 330,
- *             Boston,  MA  02111-1307, USA.
+ *             51 Franklin Street, Fifth Floor,
+ *             Boston, MA  02110-1301, USA.
  */
 
 #define ENTRY_END  INT_MAX
 
-STATIC tForState*  pFS;  /* Current "FOR" information (state) */
+static tForState*  pFS;  /* Current "FOR" information (state) */
 
+tSCC zNoEnd[] = "%s ERROR:  FOR loop `%s' does not end\n";
 
-STATIC ag_bool nextDefinition( ag_bool invert, tDefEntry** ppList );
-STATIC int
+/* = = = START-STATIC-FORWARD = = = */
+/* static forward declarations maintained by :mkfwd */
+static ag_bool
+nextDefinition( ag_bool invert, tDefEntry** ppList );
+
+static int
 doForByStep( tTemplate* pT,
              tMacro*    pMac,
              tDefEntry* pFoundDef );
-STATIC int
+
+static int
 doForEach( tTemplate*   pT,
            tMacro*      pMac,
            tDefEntry*   pFoundDef );
 
+static void
+load_ForIn( tCC* pzSrc, int srcLen, tTemplate* pT, tMacro* pMac );
+/* = = = END-STATIC-FORWARD = = = */
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *
  *  Operational Functions */
@@ -236,7 +246,7 @@ ag_scm_for_sep( SCM obj )
 }
 
 
-STATIC ag_bool
+static ag_bool
 nextDefinition( ag_bool invert, tDefEntry** ppList )
 {
     ag_bool     haveMatch = AG_FALSE;
@@ -289,7 +299,7 @@ nextDefinition( ag_bool invert, tDefEntry** ppList )
 }
 
 
-STATIC int
+static int
 doForByStep( tTemplate* pT,
              tMacro*    pMac,
              tDefEntry* pFoundDef )
@@ -421,7 +431,7 @@ doForByStep( tTemplate* pT,
     return loopCt;
 }
 
-STATIC int
+static int
 doForEach( tTemplate*   pT,
            tMacro*      pMac,
            tDefEntry*   pFoundDef )
@@ -484,9 +494,7 @@ doForEach( tTemplate*   pT,
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-tSCC zNoEnd[] = "%s ERROR:  FOR loop `%s' does not end\n";
-
-STATIC void
+static void
 load_ForIn( tCC* pzSrc, int srcLen, tTemplate* pT, tMacro* pMac )
 {
     char* pzName = pT->pzTemplText + pMac->ozName;
@@ -639,6 +647,9 @@ load_ForIn( tCC* pzSrc, int srcLen, tTemplate* pT, tMacro* pMac )
  *  and @code{for-index} functions are disabled outside of the range
  *  of a @code{FOR} block.
  *
+ *  @strong{Also:} the @code{<value-name>} must be a single level name,
+ *  not a compound name (@pxref{naming values}).
+ *
  *  @example
  *  [+FOR var (for-from 0) (for-to <number>) (for-sep ",") +]
  *  ... text with @code{var}ious substitutions ...[+
