@@ -1,12 +1,12 @@
 
 /*
- *  $Id: funcIf.c,v 1.26 2004/02/02 03:31:50 bkorb Exp $
+ *  $Id: funcIf.c,v 4.4 2005/06/07 22:26:43 bkorb Exp $
  *
  *  This module implements the _IF text function.
  */
 
 /*
- *  AutoGen copyright 1992-2004 Bruce Korb
+ *  AutoGen copyright 1992-2005 Bruce Korb
  *
  *  AutoGen is free software.
  *  You may redistribute it and/or modify it under the terms of the
@@ -21,15 +21,34 @@
  *  You should have received a copy of the GNU General Public License
  *  along with AutoGen.  See the file "COPYING".  If not,
  *  write to:  The Free Software Foundation, Inc.,
- *             59 Temple Place - Suite 330,
- *             Boston,  MA  02111-1307, USA.
+ *             51 Franklin Street, Fifth Floor,
+ *             Boston, MA  02110-1301, USA.
  */
 
 tSCC zNoIfEnd[]  = "%s ERROR:  cannot find ENDIF\n\t'%s'\n";
 tSCC zNoIfExpr[] = "expressionless IF";
 
-STATIC ag_bool eval_true( void );
 
+typedef struct if_stack tIfStack;
+struct if_stack {
+    tMacro*  pIf;
+    tMacro*  pElse;
+};
+
+static tIfStack  current_if;
+static tLoadProc mLoad_Elif, mLoad_Else;
+
+/* = = = START-STATIC-FORWARD = = = */
+/* static forward declarations maintained by :mkfwd */
+static ag_bool
+eval_true( void );
+
+static tMacro*
+mLoad_Elif( tTemplate* pT, tMacro* pMac, tCC** ppzScan );
+
+static tMacro*
+mLoad_Else( tTemplate* pT, tMacro* pMac, tCC** ppzScan );
+/* = = = END-STATIC-FORWARD = = = */
 
 /*
  *  eval_true - should a string be interpreted as TRUE?
@@ -42,7 +61,7 @@ STATIC ag_bool eval_true( void );
  *  4.  For its length or its first five characters (whichever is less)
  *      it matches the string "false"
  */
-STATIC ag_bool
+static ag_bool
 eval_true( void )
 {
     ag_bool needFree;
@@ -238,16 +257,6 @@ mFunc_While( tTemplate* pT, tMacro* pMac )
 }
 
 
-typedef struct if_stack tIfStack;
-struct if_stack {
-    tMacro*  pIf;
-    tMacro*  pElse;
-};
-
-STATIC tIfStack  current_if;
-STATIC tLoadProc mLoad_Elif, mLoad_Else;
-
-
 /*=macfunc ELIF
  *
  *  what:   Alternate Conditional Template Block
@@ -259,7 +268,7 @@ STATIC tLoadProc mLoad_Elif, mLoad_Else;
  *    @code{IF} function.  Its expression argument is evaluated as are
  *    the arguments to @code{IF}.  For a complete description @xref{IF}.
 =*/
-STATIC tMacro*
+static tMacro*
 mLoad_Elif( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 {
     if ((int)pMac->res == 0)
@@ -285,7 +294,7 @@ mLoad_Elif( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
  *    It denotes the start of an alternate template block for
  *    the @code{IF} function.  For a complete description @xref{IF}.
 =*/
-STATIC tMacro*
+static tMacro*
 mLoad_Else( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 {
     /*
